using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Diagram.Controls;

namespace SyncfusionTest
{
    public partial class Form1 : Form
    {

        protected Tools activeTool;
        private MyLinkSupport myLinkSupport;

        private MyCirclePort sourceSymbolPort = null;
        private MyCirclePort targetSymbolPort = null;

        public Form1()
        {
            InitializeComponent();
            this.diagram1.MouseDown += new MouseEventHandler(diagram1_MouseDown);
            this.diagram1.MouseMove += new MouseEventHandler(diagram1_MouseMove);

            this.myLinkSupport = new MyLinkSupport();

        }

        void diagram1_MouseMove(object sender, MouseEventArgs e)
        {
            Point ptScreen = new Point(e.X, e.Y);

            if (this.activeTool == Tools.AddLinkTool)
            {
                Port portHit = this.diagram1.Controller.View.GetPortAt(ptScreen, Constants.NodeConstants.PortCurveSlope);
                if ((portHit != null) && (portHit is MyCirclePort))
                {
                    this.diagram1.Controller.Cursor = Cursors.Arrow;
                }
                else if (null != this.sourceSymbolPort)
                {
                    this.diagram1.Controller.Cursor = Cursors.Arrow;
                }
                else
                {
                    this.diagram1.Controller.Cursor = Cursors.Arrow;
                }
                if (null != this.sourceSymbolPort)
                {
                    this.DrawTrackLines(e.X, e.Y);
                }
            }
         }
        private void DrawTrackLines(int x, int y)
        {
            if (activeTool == Tools.AddLinkTool)
            {
                diagram1.Refresh();

                PointF viewpts1 = new PointF();
                viewpts1 = diagram1.View.WorldToView(new Point((int)this.sourceSymbolPort.X, (int)this.sourceSymbolPort.Y));
                Point endPoint = new Point(x, y);

                MyCirclePort dummyPort = this.diagram1.View.GetPortAt(endPoint, 5) as MyCirclePort;
                if (dummyPort == null)
                {
                    MyCirclePort port = new MyCirclePort(endPoint);
                    dummyPort = port;
                }
                CompassHeading[] headings = new CompassHeading[2];
                headings = myLinkSupport.CalculateCompassHeadings(this.sourceSymbolPort, dummyPort);
                CompassHeading startHeading = headings[0];

                OrthogonalLine orthoLine = new OrthogonalLine();

                if ((headings[0] == CompassHeading.None) || (headings[1] == CompassHeading.None))
                    Geometry.CalcEndpointDirections(new PointF(0, 0), endPoint, out headings[0], out headings[1]);

                PointF[] pt = Geometry.CalcOrthogonalPoints(viewpts1, startHeading, endPoint, headings[1], orthoLine.PaddingLeft, orthoLine.PaddingRight, orthoLine.PaddingTop, orthoLine.PaddingBottom);

                Point[] devpts = new Point[pt.Length];
                diagram1.View.ViewToDevice(pt, out devpts);

                diagram1.View.DrawTrackingLines(devpts);
            }
        }

        private void ConnectNodes()
        {
            if ((null != this.sourceSymbolPort) && (null != this.targetSymbolPort) &&
                this.diagram1.Model.Nodes.Contains(this.sourceSymbolPort.Parent as MySymbol) &&
                this.diagram1.Model.Nodes.Contains(this.targetSymbolPort.Parent as MySymbol))
            {
                PointF[] pts = new PointF[2];
                pts[0] = new PointF(this.sourceSymbolPort.X, this.sourceSymbolPort.Y);
                pts[1] = new PointF(this.targetSymbolPort.X, this.targetSymbolPort.Y);

                LinkCmd linkCommand = new LinkCmd(this.sourceSymbolPort, this.targetSymbolPort, pts);

                CompassHeading[] compassHeading = myLinkSupport.CalculateCompassHeadings(this.sourceSymbolPort, this.targetSymbolPort);

                MyLink ortholink = new MyLink(this.sourceSymbolPort.Location,
                    compassHeading[0], this.targetSymbolPort.Location, compassHeading[1]);

                linkCommand.Link = ortholink;
                linkCommand.Link.EditStyle.AllowSelect = true;

                if (myLinkSupport.CheckForDuplicateConnections(this.sourceSymbolPort.Parent as MySymbol, this.targetSymbolPort.Parent as MySymbol))
                {
                    this.diagram1.Controller.ExecuteCommand(linkCommand);
                    this.diagram1.Controller.SendToBack();
                }
            }
            ClearConnectorObjects();
        }

        private void ClearConnectorObjects()
        {
            this.sourceSymbolPort = null;
            this.targetSymbolPort = null;
            this.diagram1.Controller.Cursor = Cursors.Arrow;
            this.diagram1.SelectionList.Clear();
        }

        void diagram1_MouseDown(object sender, MouseEventArgs e)
        {
            if (activeTool == Tools.AddLinkTool)
            {
                Point ptScreen = new Point(e.X, e.Y);
                MyCirclePort portHit = this.diagram1.Controller.View.GetPortAt(ptScreen, Constants.NodeConstants.PortCurveSlope) as MyCirclePort;

                if (portHit != null)
                {
                    if (this.sourceSymbolPort == null)
                    {
                        this.sourceSymbolPort = portHit;
                    }
                    else
                    {
                        this.targetSymbolPort = portHit;
                        ConnectNodes();
                    }
                }
            }
        }

        public enum Tools
        {
            MoveTool,
            AddNodeTool,
            AddLinkTool
        }
        private void button1_Click(object sender, EventArgs e)
        {
            //MySymbol testsymbol = new MySymbol();
            //InsertSymbolTool tool = new InsertSymbolTool( "InsertSymbol", typeof(MySymbol));
            //this.diagram1.Controller.RegisterTool(tool);
            //tool.Symbol = testsymbol;
            //this.diagram1.Controller.ActivateTool(tool);
            //this.diagram1.Controller.Model.UpdateViews();

            InsertNodesCmd cmd = new InsertNodesCmd();
            MySymbol testsymbol = new MySymbol();
            cmd.Nodes.Add(testsymbol);
            cmd.Location = new PointF(50, 60);
            this.diagram1.ExecuteCommand(cmd);

        }

        private void button_Exit_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        private void button_addConnector_Click(object sender, EventArgs e)
        {
            ActivateTool(Tools.AddLinkTool);
        }
        public void ActivateTool(Tools tool)
        {
            this.diagram1.Controller.Cursor = Cursors.Arrow;
            this.activeTool = tool;
            this.diagram1.Refresh();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            NodeCollection selectednodes = this.diagram1.SelectionList;
            if (selectednodes.First is Symbol)
            {
                Symbol node = (selectednodes.First) as Symbol;
                PointF pt = new PointF(node.X, node.Y + node.Height / 2);

                CirclePort port = new CirclePort(pt);

                node.Ports.Add(port);
                port.Location = pt;
                this.diagram1.Refresh();
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            NodeCollection selectednodes = this.diagram1.SelectionList;
            if (selectednodes.First is Symbol)
            {
                Symbol node = (selectednodes.First) as Symbol;
                PointF pt = new PointF(node.X + node.Width, node.Y + node.Height / 2);
                CirclePort port = new CirclePort(pt);
                node.Ports.Add(port);
                port.Location = pt;
                this.diagram1.Refresh();
            }
        }
    }


}