using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Drawing;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Diagram;
using View = Syncfusion.Windows.Forms.Diagram.View;

namespace SyncfusionTest
{
	/// <summary>
	/// This class is SF MoMDiagramController. 
	/// Overriding the Mouse Events and passing the Mouse events only when it has focus.
	/// SF Patch.
	/// </summary>
	[
	Serializable,
		ToolboxItem(false)
	]
	public class MoMDiagramController : DiagramController
	{
		Stack redo = null,undo = null;

		#region Constructors

		/// <summary>
		/// Default constructor.
		/// </summary>
		public MoMDiagramController()
		{
		}

		/// <summary>
		/// Constructs a DiagramController and attaches it to get specified view.
		/// </summary>
		/// <param name="view"></param>
		public MoMDiagramController(View view) : base(view)
		{
		}

		#endregion

		#region Mouse event overrides

		protected override void OnMouseDown(object sender, MouseEventArgs evtArgs)
		{
//			if(this.ParentControl.ContainsFocus == true)
//				base.OnMouseDown (sender, evtArgs);

			if (evtArgs.Button == MouseButtons.Left)
			{
			// Check to see if there is a port at this location
			Port port = this.View.GetPortAt(new Point (evtArgs.X, evtArgs.Y), 0);

				// Activate the link tool if there is a port and it is valid
			if (port != null)
			{
				
				if (port.GetType() == typeof(CirclePort))
			{
			this.ActivateTool("LinkTool");
		}
		}
		}

		base.OnMouseDown(sender, evtArgs);
		}

		
		
		//Commented this code as it was causing an error, i.e. The Add node cursor was not 
		//Visible for the first time we add a node on a diagram after Loading the diagram or
		//after Selected page changed of the Tab page.
		protected override void OnMouseMove(object sender, MouseEventArgs evtArgs)
		{
			if(this.ParentControl.ContainsFocus == true)
				base.OnMouseMove (sender, evtArgs);
		}

		protected override void OnMouseUp(object sender, MouseEventArgs evtArgs)
		{
			if(this.ParentControl.ContainsFocus == true)
				base.OnMouseUp (sender, evtArgs);
		}

		protected void OnMouseLeave(object sender, MouseEventArgs evtArgs)
		{
			if(this.ParentControl.ContainsFocus == true)
				base.OnMouseUp (sender, evtArgs);
		}

		#endregion

		#region Initialize

		/// <summary>
		/// Initializes the controller for the given view.
		/// </summary>
		/// <param name="view">The view object to attach to the controller.</param>
		/// <remarks>
		/// This method initializes the controller and registers all of the
		/// tools in the Essential Diagram library.
		/// </remarks>
		public override void Initialize(View view)
		{
			base.Initialize(view);

            //RegisterTool(new MoveTool());
            //RegisterTool(new ResizeTool());
            //RegisterTool(new RotateTool());
            //RegisterTool(new LineTool());
            //RegisterTool(new OrthogonalLineTool());
            //RegisterTool(new PolyLineTool());
            //RegisterTool(new RectangleTool());
            //RegisterTool(new RoundRectTool());
            //RegisterTool(new ArcTool());
            //RegisterTool(new PolygonTool());
            //RegisterTool(new EllipseTool());
            //RegisterTool(new CurveTool());
            //RegisterTool(new ClosedCurveTool());
            //RegisterTool(new PortTool());
            //RegisterTool(new LinkTool());
            //RegisterTool(new RichTextTool());
            //RegisterTool(new ZoomTool());
            //RegisterTool(new PanTool());
            //RegisterTool(new GroupTool());
            //RegisterTool(new UngroupTool());
            //RegisterTool(new SelectTool());
            //RegisterTool(new MetafileTool());
            //RegisterTool(new BitmapTool());
            //RegisterTool(new VertexEditTool());
            //RegisterTool(new ControlNodeTool());
            //RegisterTool(new MoMLineSegmentTool());
		}

		#endregion

		public Stack UndoStack
		{
			get
			{ 
				Type myType = typeof(Syncfusion.Windows.Forms.Diagram.Controller);
				System.Reflection.FieldInfo fields1 = myType.GetField("undoStack", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.GetField);
				if(fields1!=null)
					undo = (Stack)fields1.GetValue(this);
				return this.undo; 
			}
		}

		public Stack RedoStack
		{
			get 
			{
				Type myType = typeof(Syncfusion.Windows.Forms.Diagram.Controller);
				System.Reflection.FieldInfo fields = myType.GetField("redoStack", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.GetField);
			
				if(fields!=null)
					redo = (Stack)fields.GetValue(this); 
				return this.redo; 
			}
		}
	}
}
