using System;
using System.Collections.Generic;
using System.Text;

using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.Serialization;
using Syncfusion.Windows.Forms.Diagram;

namespace SyncfusionTest
{
    [Serializable()]
	public class MyCirclePort : MoveablePort
	{
		private const float RADIUS = 5.0f;
		private readonly PointF LOCATION = new PointF(0,0);

		private float height;
		private float width;
		

		private FillStyle fillstyle = null;

		#region Constructors

		public MyCirclePort()
		{
			this.fillstyle = new FillStyle(this);
			this.Location = LOCATION;
			this.grfxPath = null;


			this.EditStyle.AllowSelect = false;
			this.EditStyle.AllowMove = false;
		}
		
		//For the Top and Bottom Ports
		public MyCirclePort(PointF location)
		{
			this.fillstyle = new FillStyle(this);

			this.Location = new PointF((int)location.X-48,(int)location.Y);
			this.height = 11;
			this.width = 106;

			this.grfxPath = new GraphicsPath();
			grfxPath.StartFigure();
			System.Drawing.Rectangle rect = new System.Drawing.Rectangle ((int)this.Location.X-48, (int)this.Location.Y, 106,11);
			grfxPath.AddRectangle(rect) ;			

			grfxPath.CloseFigure();
			
			this.EditStyle.AllowMove = false;
			this.EditStyle.AllowSelect = false;
		}
		
		//For the left and right ports
		public MyCirclePort(PointF location, float height)
		{
			
			this.fillstyle = new FillStyle(this);

			this.Location =new PointF(location.X, (int)(location.Y - height/2));
			this.width = 11;
			this.height = height;

			this.grfxPath = new GraphicsPath();
			grfxPath.StartFigure();
			
			System.Drawing.Rectangle  rect = new System.Drawing.Rectangle ((int)this.Location.X, (int)(this.Location.Y - height/2),11,(int)height);
			grfxPath.AddRectangle(rect) ;

			grfxPath.CloseFigure();
			
			this.EditStyle.AllowMove = false;
			this.EditStyle.AllowSelect = false;
		}

		public MyCirclePort(MyCirclePort src) : base(src)
		{
			this.width  = src.width;
			this.height = src.height;
		}

		#endregion

		#region Custom Serialization

		protected MyCirclePort(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			try
			{
				this.fillstyle = new FillStyle(this);
				this.width = (float)info.GetValue("width", this.width.GetType());
				this.height = (float) info.GetValue("height", this.height.GetType()); 
			}
			catch(Exception ex)
			{	
				// TODO: SyncFusion Error: waiting for their reply. Currently eating the exception
				Debug.WriteLine(DateTime.Now.Second + ":" + DateTime.Now.Millisecond + ": " + ex.Message);
			}
		}

		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.SetType(this.GetType());
			base.GetObjectData(info, context);
			info.AddValue("width" , this.width , this.width.GetType());
			info.AddValue("height" , this.height , this.height.GetType());

		}

		#endregion

		#region Properties

		/// <summary>
		/// X-coordinate of the port's location.
		/// </summary>
		/// <remarks>
		/// Specified in world coordinates.
		/// </remarks>
		[
		Browsable(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
		Category("Location"),
		Description("Position of the port along the X axis.")
		]
		public override float X
		{
			get
			{
				return this.Location.X;
			}
			set
			{
				this.Location = new PointF(value, this.Location.Y);
			}
		}

		/// <summary>
		/// Y-coordinate of the port's location.
		/// </summary>
		/// <remarks>
		/// Specified in world coordinates.
		/// </remarks>
		[
		Browsable(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
		Category("Location"),
		Description("Position of the port along the Y axis.")
		]
		public override float Y
		{
			get
			{
				return this.Location.Y;
			}
			set
			{
				this.Location = new PointF(this.Location.X, value);
			}
		}

		/// <summary>
		/// Size of the port.
		/// </summary>
		/// <remarks>
		/// <seealso cref="Syncfusion.Windows.Forms.Diagram.Port.Bounds"/>
		/// </remarks>
		[
		Browsable(false),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public override SizeF Size
		{
			get
			{
				return new SizeF(this.Width , this.Height);
			}
			set
			{
				Size = value;
			}
		}

		/// <summary>
		/// Width of the port.
		/// </summary>
		/// <remarks>
		/// Specified in world coordinates.
		/// </remarks>
		[
		Browsable(false),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public override float Width
		{
			get
			{
				return this.width;
			}
			set
			{
				this.width = value;
			}
		}

		/// <summary>
		/// Height of the port.
		/// </summary>
		/// <remarks>
		/// Specified in world coordinates.
		/// </remarks>
		[
		Browsable(false),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public override float Height
		{
			get
			{
				return this.height;
			}
			set
			{
				this.height= value;
			}
		}


		[
		Browsable(false),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public override RectangleF Bounds
		{
			get
			{
				PointF ptPort = this.Location;
				return new RectangleF((ptPort.X - this.Width/2), (ptPort.Y - this.Height/2), this.Width, this.Height);
			}
			set
			{
				this.Location = value.Location;
				this.Size = value.Size;
			}
		}

		

		[Browsable(true),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
		TypeConverter(typeof(FillStyleConverter)),Category("Appearance"),
		Description("Properties of the brush used for filling rectangle.")]
	
		public FillStyle FillStyle
		{
			get
			{
				return this.fillstyle;
			}
		}

		#endregion

		#region Base Overrides

		public override object Clone()
		{
			return new MyCirclePort(this);
		}

		public override void Draw(Graphics grfx)
		{
			if(this.Visible && this.grfxPath != null)
			{
				grfx.Transform = Global.ViewMatrix;
				PointF[] worldOffset = new PointF[]{new PointF(0.0f, 0.0f)};

				Matrix xform = Global.MatrixStack.Push(this.LocalTransform, MatrixOrder.Prepend);
				xform.TransformPoints(worldOffset);
				Matrix worldOffsetXform = new Matrix(1.0f, 0.0f, 0.0f, 1.0f, worldOffset[0].X, worldOffset[0].Y);

				GraphicsPath grfxPath = (GraphicsPath) this.grfxPath.Clone();
				grfxPath.Transform(worldOffsetXform);

				Pen pen = this.LineStyle.CreatePen();
				grfx.DrawPath(pen, grfxPath);
				RectangleF rect = new RectangleF(100,100,120,80);
				Brush brush = this.fillstyle.CreateBrush(grfx,rect);
				grfx.FillPath(brush, grfxPath);

				grfxPath.Dispose();
				pen.Dispose();
				brush.Dispose();
				Global.MatrixStack.Pop();
			}
		}

		public override void SetPropertyValue(string propertyName, object val)
		{
			try
			{
				base.SetPropertyValue (propertyName, val);
			}
			catch(Exception ex)
			{	
				// TODO: SyncFusion Error: waiting for their reply. Currently eating the exception
				Debug.WriteLine(DateTime.Now.Second + ":" + DateTime.Now.Millisecond + ": " + ex.Message);
			}
			
			PropertyEventArgs evtArgs = null;
			evtArgs = new PropertyEventArgs(this, propertyName, 0, val);
			if (this.Parent != null)
			{
				IDispatchNodeEvents evtDispatcher = this.Parent.GetService(typeof(IDispatchNodeEvents)) as IDispatchNodeEvents;
				if (evtDispatcher != null)
				{
					evtDispatcher.PropertyChanged(evtArgs);
				}
			}
		}

		#endregion

	}
}
