using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.Serialization;
using System.Security.Permissions;

using Syncfusion.Windows.Forms.Diagram;
using Rectangle = Syncfusion.Windows.Forms.Diagram.Rectangle;

namespace SyncfusionTest
{
    /// <summary>
    /// Derived From Syncfusion Symbol Class to implement the basic structure
    /// of the Symboloperation. 
    /// </summary>
    [Serializable]
    public class MySymbol : Symbol
    {
       #region Private members

        private Rectangle nodeSymbolHeader = null;
        private Rectangle nodeSymbolBody = null;
        private Rectangle nodeSymbolFooter = null;
        private Node dataNode = null;
        private MyCirclePort topPort = null;
        private MyCirclePort bottomPort = null;
        private MyCirclePort leftPort = null;
        private MyCirclePort rightPort = null;
        private float translateX;
        private float translateY;

        #endregion Private members


        # region Symbol Construction

        public MySymbol()
        {
            this.dataNode = new Node();
            ConstructSymbol();
        }
        private void ConstructSymbol()
        {
           SetupNodeSymbolBackground();
           SetAnchorPorts();
           this.nodeSymbolHeader.FillStyle.Color = Color.Gray;
           this.nodeSymbolBody.FillStyle.Color = Color.Silver;
           this.nodeSymbolFooter.FillStyle.Color = Color.SteelBlue;
           this.Width = 120F;
           AdjustHeight();
        }
        public void AdjustHeight()
        {
            this.EditStyle.AllowResize = true;
            this.EditStyle.AllowMove = true;
        
            this.nodeSymbolBody.Height = 16;

            this.nodeSymbolFooter.Y = this.Y + this.nodeSymbolHeader.Height
                + this.nodeSymbolBody.Height;

            this.topPort.EditStyle.AllowMove = true;
            this.topPort.X = this.X + this.nodeSymbolHeader.Width / 2;
            this.topPort.Y = this.nodeSymbolHeader.Y - 1;
            this.topPort.EditStyle.AllowMove = false;

            this.bottomPort.EditStyle.AllowMove = true;
            this.bottomPort.X = this.X + this.nodeSymbolFooter.Width / 2;
            this.bottomPort.Y = this.Y + this.Height;
            this.bottomPort.EditStyle.AllowMove = false;

            this.leftPort.EditStyle.AllowMove = true;
            this.leftPort.Y = this.Y + this.Height / 2 - 2;
            this.leftPort.Height = this.nodeSymbolBody.Height;
            this.leftPort.EditStyle.AllowMove = false;

            this.rightPort.EditStyle.AllowMove = true;
            this.rightPort.Y = this.Y + this.Height / 2 - 2;
            this.rightPort.Height = this.nodeSymbolBody.Height;
            this.rightPort.EditStyle.AllowMove = false;

           this.EditStyle.AllowResize = false;
       }
       /// <summary>
        /// Called when deserialization is complete.
        /// </summary>
        /// <param name="sender">Object performing the deserialization</param>
        protected override void OnDeserialization(object sender)
        {
            base.OnDeserialization(sender);

            this.nodeSymbolHeader = this.GetChildByName(Constants.NodeConstants.nodeSymbolHeader) as Rectangle;
            this.nodeSymbolBody = this.GetChildByName(Constants.NodeConstants.nodeSymbolBody) as Rectangle;
            this.nodeSymbolFooter = this.GetChildByName(Constants.NodeConstants.nodeSymbolFooter) as Rectangle;
            this.topPort = this.GetChildByName(Constants.NodeConstants.topPort) as MyCirclePort;
            this.bottomPort = this.GetChildByName(Constants.NodeConstants.bottomPort) as MyCirclePort;
            this.leftPort = this.GetChildByName(Constants.NodeConstants.leftPort) as MyCirclePort;
            this.rightPort = this.GetChildByName(Constants.NodeConstants.rightPort) as MyCirclePort;
        }
        // Serialization constructor for symbols.		
        protected MySymbol(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            try
            {
                //Here we are taking the dataNode values of the copied object, but the NodeType, value
                //of the nodeSymbol is representing the dataNode.Type, hence will give the current value of the
                //Node type, and will not tell its value at the time of copy, Thus we have to take that value 
                //Seperatly, and we are setting that value in the UpdateNodeSymbol function which is called after
                //a node is inserted in the diagram.
                this.dataNode = (Node)info.GetValue(Constants.NodeConstants.dataNode, typeof(Node));
                this.translateX = (float)info.GetDouble(Constants.NodeConstants.translateX);
                this.translateY = (float)info.GetDouble(Constants.NodeConstants.translateY);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(DateTime.Now.Second + ":" + DateTime.Now.Millisecond + ": " + ex.Message);
            }
        }

        /// Populates a SerializationInfo with the data needed to
        /// serialize the target object.
        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = false)]
        protected override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.SetType(this.GetType());
            base.GetObjectData(info, context);
            info.AddValue(Constants.NodeConstants.nodeSymbolHeader, this.nodeSymbolHeader);
            info.AddValue(Constants.NodeConstants.nodeSymbolBody, this.nodeSymbolBody);
            info.AddValue(Constants.NodeConstants.nodeSymbolFooter, this.nodeSymbolFooter);
            info.AddValue(Constants.NodeConstants.dataNode, this.dataNode, this.dataNode.GetType());
            info.AddValue(Constants.NodeConstants.topPort, this.topPort);
            info.AddValue(Constants.NodeConstants.bottomPort, this.bottomPort);
            info.AddValue(Constants.NodeConstants.leftPort, this.leftPort);
            info.AddValue(Constants.NodeConstants.rightPort, this.rightPort);
            info.AddValue(Constants.NodeConstants.translateX, this.translateX);
            info.AddValue(Constants.NodeConstants.translateY, this.translateY);
        }

        # endregion
        

        #region Setup functions

        private void SetupNodeSymbolBackground()
        {
            this.nodeSymbolHeader = new Rectangle(0, 0, 120F, 5);
            this.nodeSymbolBody = new Rectangle(0, 5, 120F, 16);
            this.nodeSymbolFooter = new Rectangle(0, 21, 120F, 20);
            this.AppendChild(this.nodeSymbolHeader);
            this.AppendChild(this.nodeSymbolBody);
            this.AppendChild(this.nodeSymbolFooter);
        }
        private void SetAnchorPorts()
        {
            this.CenterPort.Enabled = false;
            PointF portPoint = new PointF((float)(this.nodeSymbolHeader.Width / 4 - 2.5), -3);
            this.topPort = new MyCirclePort(portPoint);
            portPoint.X = (float)(this.nodeSymbolFooter.Width / 4 - 2.5);
            portPoint.Y = (float)(this.Height / 2 - 2.5);
            this.bottomPort = new MyCirclePort(portPoint);

            portPoint.X = this.nodeSymbolBody.X - 3;
            portPoint.Y = this.Height / 4 - 2;
            this.leftPort = new MyCirclePort(portPoint, this.nodeSymbolBody.Height);
            portPoint.X = this.nodeSymbolBody.Width / 2 - 3;
            portPoint.Y = this.Height / 4 - 2;
            this.rightPort = new MyCirclePort(portPoint, this.nodeSymbolBody.Height);


            this.topPort.Visible = false;
            this.topPort.Name = Constants.NodeConstants.topPort;
            this.topPort.EditStyle.AllowMove = false;
            this.topPort.Enabled = true;

            this.bottomPort.Visible = false;
            this.bottomPort.Name = Constants.NodeConstants.bottomPort;
            this.bottomPort.EditStyle.AllowMove = false;
            this.bottomPort.Enabled = true;

            this.leftPort.Visible = false;
            this.leftPort.Name = Constants.NodeConstants.leftPort;
            this.leftPort.EditStyle.AllowMove = false;
            this.leftPort.Enabled = true;

            this.rightPort.Visible = false;
            this.rightPort.Name = Constants.NodeConstants.rightPort;
            this.rightPort.EditStyle.AllowMove = false;
            this.rightPort.Enabled = true;

            this.AppendChild(this.topPort);
            this.AppendChild(this.bottomPort);
            this.AppendChild(this.leftPort);
            this.AppendChild(this.rightPort);
        }
        #endregion


        #region Public Properties
        public Rectangle NodeSymbolHeader
        {
            get { return this.nodeSymbolHeader; }
            set { this.nodeSymbolHeader = value; }
        }

        public Rectangle NodeSymbolBody
        {
            get { return this.nodeSymbolBody; }
            set { this.nodeSymbolBody = value; }
        }

        public Rectangle NodeSymbolFooter
        {
            get { return this.nodeSymbolFooter; }
            set { this.nodeSymbolFooter = value; }
        }

        public Node DataNode
        {
            get { return this.dataNode; }
            set { this.dataNode = value; }
        }

        public MyCirclePort TopPort
        {
            get { return this.topPort; }
            set { this.topPort = value; }
        }

        public MyCirclePort BottomPort
        {
            get { return this.bottomPort; }
            set { this.bottomPort = value; }
        }
        public MyCirclePort LeftPort
        {
            get { return this.leftPort; }
            set { this.leftPort = value; }
        }

        /// <summary>
        /// Gets or sets the reference of rightPort
        /// </summary>
        public MyCirclePort RightPort
        {
            get { return this.rightPort; }
            set { this.rightPort = value; }
        }
       #endregion
    }
}