using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Security.Permissions;

namespace SyncfusionTest
{
    [Serializable]
	public enum NodeCareZoneType
	{
		Primary,
		Secondary,
		Invalid
	}

	/// <summary>
	/// Specifies the different types of Nodes used in the a Map.
	/// </summary>
	[Serializable]
	public enum NodeType
	{
		Standard,
		Referral,
		PageLink,
	}

	/// <summary>
	/// Summary description for Node.
	/// </summary>
	[Serializable]
	public class Node : ISerializable
	{

		#region Private Members

		private NodeType nodeType;

		
		#endregion Private Members

		# region Constructors
		/// <summary>
		/// Default Constructor
		/// </summary>
		public Node()
		{
			this.nodeType = NodeType.Standard;
		}
		
		/// <summary>
		/// Constructor used in cut/copy and paste.
		/// </summary>
		/// <remarks>
		/// Remember to update the <see cref="GetObjectData"/> whenever there is a change made to this function or vice versa.
		/// </remarks>
		/// <param name="info">Xml of node from the page Xml.</param>
		/// <param name="context">ID of the page that contains the node.</param>
		protected Node(SerializationInfo info, StreamingContext context) : this()
		{
		}
		
		#endregion
			
		#region Public properties

		/// <summary>
		/// Gets or sets the node type value
		/// </summary>
		public NodeType Type
		{
			// TODO: Implement Node Type after getting data from the baseXml object
			get
			{
				return this.nodeType;
			}
			set
			{
				if(value != this.nodeType)
				{
					this.nodeType = value;
				}
				
			}
		}

		#endregion

		#region Public Functions

		/// <summary>
		/// Clears all child nodes.
		/// </summary>

		[SecurityPermission(SecurityAction.Demand, SerializationFormatter = false)]
		public void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.SetType(this.GetType());
			info.AddValue("NodeType", this.nodeType);
		}

		#endregion


		/// <summary>
		/// Creates an empty node for first use. 
		/// Ensures that base xml data is properly initialized based on Node type.
		/// </summary>
		// Creates a new object that is a copy of the current instance.	
		// An override of the Clone method is required for clipboard copy/paste.
		public Node Clone()
		{
			using (MemoryStream stream = new MemoryStream())
			{
				BinaryFormatter bFormatter = new BinaryFormatter();
				bFormatter.Serialize(stream, this);

				stream.Seek(0, SeekOrigin.Begin);
				Node clone = (Node)bFormatter.Deserialize(stream);
				return clone;
			}
		}
	}
}